## **********************************************************************
## Custom R function to calculate bounded HPD
## **********************************************************************

## sample is the full MCMC posterior sample (of lambda)
## lower is a lower bound (for lambda this is 0, but can take -Inf)
## upper is upper bound (for lambda this is 1, but can take Inf)
## mode = TRUE if you want the posterior mode (highest posterior probability)
## HPDcoverage = percentage coverage by kernel method, or FALSE if no interval
## wanted
## codaHPD = TRUE if interval wanted with same HPDcoverage amount or a number
## if a different amount wanted. Coda HPDs can not equal 0 or 1 when these 
## are the bounds, so its purpose here is for comparison with custom HPS
## coverage.
## n is the number of points that determine the resolution of the density
## function

estimateFromBoundedPosterior <- function(sample, lower = 0, upper = 1, 
										 mode = TRUE, HPDcoverage = 0.95, 
										 codaHPD = TRUE, n = 1024) {
    ## use method from coda::densplot of reflecting sample
    reflectedSample <- c(if(lower > -Inf) lower - (sample-lower) 
    						else numeric(), 
    					 sample,
                         if(upper < Inf) upper + (upper-sample) 
                         	else numeric() )

    griddedDensityCall <- quote(density(reflectedSample, 
    									n = n, bw = "SJ"))
    if(lower > -Inf) griddedDensityCall[['from']] <- lower
    if(upper < Inf) griddedDensityCall[['to']] <- upper
    griddedDensity <- eval(griddedDensityCall)

    if(mode) {
        iMax <- which.max(griddedDensity$y)
        argMaxX <- griddedDensity$x[iMax] ## this is the value of the mode, 
        							   ## to within precision of n 
        							   ## (default 1024 points)
    } else {
        argMaxX <- NULL
    }
    if(is.numeric(HPDcoverage)) {

        ## Normalize denisty function 
        estimatedPDF <- (griddedDensity$y / sum(griddedDensity$y))
        
        alpha <- 1-HPDcoverage
        
        funThatWeWantToSolveForZero <- function(densityLevel) {
            whichAbove <- which(estimatedPDF >= densityLevel)
            if(length(whichAbove)==0) return(1-alpha)
            if(length(whichAbove)==n) return(-alpha)
            interval <- griddedDensity$x[range(whichAbove)]
            mean(sample < interval[1] | sample > interval[2])  - alpha
        }

        HPDsolution <- uniroot(funThatWeWantToSolveForZero, 
                               lower = 0, upper = estimatedPDF[iMax])
        HPDdensityLevel <- HPDsolution$root
        whichAbove <- which(estimatedPDF > HPDdensityLevel)
        HPDinterval <- griddedDensity$x[range(whichAbove)]
    } else {
        HPDinterval <- NULL
    }

    doCodaHPD <- is.numeric(codaHPD) | identical(codaHPD, TRUE)
    if(doCodaHPD) {
        if(is.logical(codaHPD)) codaHPD <- 
        	if(is.numeric(HPDcoverage)) HPDcoverage else 0.95
        require(coda)
        codaHPDinterval <- coda::HPDinterval(as.mcmc(sample), prob = codaHPD)
    } else {
        codaHPDinterval <- NULL
    }
    
    list(mode = argMaxX, HPDinterval = HPDinterval, 
         codaHPDinterval = codaHPDinterval)
}
